
/* 
 * Copyright (c) 1993-2003,2012 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*------------------  Fichier gnr automatiquement ------------------*/
/*------------------  Ne pas diter manuellement !!! ------------------*/

#ifndef mor_MSG_LIB_H
#define mor_MSG_LIB_H

#include "portLib.h"
#include "csLib.h"
#include "posterLib.h"
 
#include "morType.h"

#define TIME_WAIT_REPLY           /*2000*/ 0
 



/*----------- Tailles des messages et boites aux lettres  ----------*/

/* Taille du buffer utilise dans la bal pour un mesage de taille size
 * (cf servMbox pour UNIX et h2rgnLib pour VXWORKS )
 */


#define BUF_SIZE(size) \
   (size + 4 - (size & 3) + sizeof(LETTER_HDR) + 8)

/**
 ** Boite aux lettres reception requetes 
 **/
#define  MOR_MBOX_NAME                  genomInstanceName("mor")

#define  MOR_MAX_RQST_SIZE              264

#define  MOR_MBOX_RQST_SIZE       \
 (BUF_SIZE(MOR_MAX_RQST_SIZE) * SERV_NMAX_RQST_ID)

/** 
 ** Taille max repliques (serveur)
 **/
#define  MOR_MAX_INTERMED_REPLY_SIZE   (sizeof(int))

#define  MOR_MAX_REPLY_SIZE             128

/** 
 ** Taille bal clients pour reception repliques 
 **/
#define  MOR_CLIENT_MBOX_REPLY_SIZE   \
((BUF_SIZE(MOR_MAX_REPLY_SIZE) \
   + BUF_SIZE(MOR_MAX_INTERMED_REPLY_SIZE)) \
 * CLIENT_NMAX_RQST_ID)

/*-------------------------- Liste des requetes  -------------------------*/

#define MOR_INIT_RQST 0
#define MOR_GETCONFIG_RQST 1
#define MOR_SETPOS_RQST 2
#define MOR_GETREF_RQST 3
#define MOR_SETREF_RQST 4
#define MOR_GETCMDPARAM_RQST 5
#define MOR_SETCMDPARAM_RQST 6
#define MOR_STOP_RQST 7
#define MOR_JOYSTICKON_RQST 8
#define MOR_TRACK_RQST 9
#define MOR_SETPOM_RQST 10
#define MOR_MOVESPINE_RQST 11
#define MOR_ABORT_RQST 12


/*---------------- PROTOTYPES DES FONCTIONS EXTERNES ------------------*/

#ifdef __cplusplus
extern "C" {
#endif

STATUS morClientInit (CLIENT_ID *pClientId);

STATUS morClientEnd (CLIENT_ID clientId);

int morAbortRqstSend (CLIENT_ID clientId, 
			   int *pRqstId,
			   int *activity,
			   int replyTimeOut);

int morAbortReplyRcv (CLIENT_ID clientId, 
			   int rqstId, 
			   int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			   int *bilan);

int morAbortRqstAndRcv (CLIENT_ID clientId, 
			     int *activity,
			     int *bilan);




extern STATUS morInitRqstSend (CLIENT_ID clientId, int *pRqstId,
					 MOR_INIT_PARAMS *(in_initParams),
					 int replyTimeOut);

extern int morInitReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int morInitRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					MOR_INIT_PARAMS *(in_initParams), 
					
					int *activity, int *bilan);

extern int morInitRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					MOR_INIT_PARAMS *(in_initParams), 
					
					int *activity,
					int *bilan);




extern STATUS morGetConfigRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int morGetConfigReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       GENPOS_CART_STATE *(out_robot),
			       int *bilan);

int morGetConfigRqstAndRcv (CLIENT_ID clientId, 
				  
				 GENPOS_CART_STATE *(out_robot),
				 int *bilan);




extern STATUS morSetPosRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 GENPOS_CART_CONFIG *(in_setPosStr),
					 int replyTimeOut);

int morSetPosReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int morSetPosRqstAndRcv (CLIENT_ID clientId, 
				 GENPOS_CART_CONFIG *(in_setPosStr), 
				 
				 int *bilan);




extern STATUS morGetRefRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int morGetRefReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       GENPOS_CART_REF *(out_ref),
			       int *bilan);

int morGetRefRqstAndRcv (CLIENT_ID clientId, 
				  
				 GENPOS_CART_REF *(out_ref),
				 int *bilan);




extern STATUS morSetRefRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 GENPOS_CART_REF *(in_ref),
					 int replyTimeOut);

int morSetRefReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int morSetRefRqstAndRcv (CLIENT_ID clientId, 
				 GENPOS_CART_REF *(in_ref), 
				 
				 int *bilan);




extern STATUS morGetCmdParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int morGetCmdParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       GENPOS_CART_CMD_PARAM *(out_cmd),
			       int *bilan);

int morGetCmdParamRqstAndRcv (CLIENT_ID clientId, 
				  
				 GENPOS_CART_CMD_PARAM *(out_cmd),
				 int *bilan);




extern STATUS morSetCmdParamRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 GENPOS_CART_CMD_PARAM *(in_cmd),
					 int replyTimeOut);

int morSetCmdParamReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int morSetCmdParamRqstAndRcv (CLIENT_ID clientId, 
				 GENPOS_CART_CMD_PARAM *(in_cmd), 
				 
				 int *bilan);




extern STATUS morStopRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int morStopReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int morStopRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS morJoystickOnRqstSend (CLIENT_ID clientId, int *pRqstId,
					 
					 int replyTimeOut);

extern int morJoystickOnReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int morJoystickOnRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					 
					
					int *activity, int *bilan);

extern int morJoystickOnRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					 
					
					int *activity,
					int *bilan);




extern STATUS morTrackRqstSend (CLIENT_ID clientId, int *pRqstId,
					 MOR_TRACK_PARAMS *(in_trackParams),
					 int replyTimeOut);

extern int morTrackReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int morTrackRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					MOR_TRACK_PARAMS *(in_trackParams), 
					
					int *activity, int *bilan);

extern int morTrackRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					MOR_TRACK_PARAMS *(in_trackParams), 
					
					int *activity,
					int *bilan);




extern STATUS morSetPomRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GENPOS_POSTER_NAME *(in_mePosterName),
					 int replyTimeOut);

extern int morSetPomReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int morSetPomRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GENPOS_POSTER_NAME *(in_mePosterName), 
					
					int *activity, int *bilan);

extern int morSetPomRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GENPOS_POSTER_NAME *(in_mePosterName), 
					
					int *activity,
					int *bilan);




extern STATUS morMoveSpineRqstSend (CLIENT_ID clientId, int *pRqstId,
					 MOR_SPINE_REF *(in_spineRef),
					 int replyTimeOut);

extern int morMoveSpineReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int morMoveSpineRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					MOR_SPINE_REF *(in_spineRef), 
					
					int *activity, int *bilan);

extern int morMoveSpineRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					MOR_SPINE_REF *(in_spineRef), 
					
					int *activity,
					int *bilan);


#ifdef __cplusplus
}
#endif
#endif /* mor_MSG_LIB_H */

